class ExtendedDatabaseWrapper(object):
    operators = {
        # For negates
        'neexact': '<> %s',
        'neiexact': '<> UPPER(%s)',
        'necontains': 'NOT LIKE %s',
        'neicontains': 'NOT LIKE UPPER(%s)',
        'neregex': '!~ %s',
        'neiregex': '!~* %s',
        'nestartswith': 'NOT LIKE %s',
        'neendswith': 'NOT LIKE %s',
        'neistartswith': 'NOT LIKE UPPER(%s)',
        'neiendswith': 'NOT LIKE UPPER(%s)',
        # For exregex
        'exregex': '~ %s',
        'exiregex': '~* %s',
        'neexregex': '!~ %s',
        'neexiregex': '!~* %s',
    }

    pattern_ops = {
       'necontains': r"NOT LIKE '%%' || {} || '%%'",
       'neicontains': r"NOT LIKE '%%' || UPPER({}) || '%%'",
       'nestartswith': r"NOT LIKE {} || '%%'",
       'neistartswith': r"NOT LIKE UPPER({}) || '%%'",
       'neendswith': r"NOT LIKE '%%' || {}",
       'neiendswith': r"NOT LIKE '%%' || UPPER({})",
    }

    regex_synonyms = {
        '\\<': '[[:<:]]',
        '\\>': '[[:>:]]',
    }



from django.apps import AppConfig

from .monkey_patches import patch_load_backend

patch_load_backend()


class LookupExtensionsConfig(AppConfig):
    name = 'lookup_extensions'

    def ready(self):
        from . import lookups  # noqa F401



class ExtendedDatabaseWrapper(object):
    operators = {
        # For negates
        'neexact': '<> %s',
        'neiexact': "NOT LIKE %s ESCAPE '\\'",
        'necontains': "NOT LIKE %s ESCAPE '\\'",
        'neicontains': "NOT LIKE %s ESCAPE '\\'",
        'neregex': 'NOT REGEXP %s',
        'neiregex': "NOT REGEXP '(?i)' || %s",
        'nestartswith': "NOT LIKE %s ESCAPE '\\'",
        'neendswith': "NOT LIKE %s ESCAPE '\\'",
        'neistartswith': "NOT LIKE %s ESCAPE '\\'",
        'neiendswith': "NOT LIKE %s ESCAPE '\\'",
        # For exregex
        'exregex': 'REGEXP %s',
        'exiregex': "REGEXP '(?i)' || %s",
        'neexregex': 'NOT REGEXP %s',
        'neexiregex': "NOT REGEXP '(?i)' || %s",
    }

    pattern_ops = {
        'necontains': r"NOT LIKE '%%' || {} || '%%' ESCAPE '\'",
        'neicontains': r"NOT LIKE '%%' || UPPER({}) || '%%' ESCAPE '\'",
        'nestartswith': r"NOT LIKE {} || '%%' ESCAPE '\'",
        'neistartswith': r"NOT LIKE UPPER({}) || '%%' ESCAPE '\'",
        'neendswith': r"NOT LIKE '%%' || {} ESCAPE '\'",
        'neiendswith': r"NOT LIKE '%%' || UPPER({}) ESCAPE '\'",
    }

    regex_synonyms = {}



from django.db import utils

from .backends.mixins import ExtendedDatabaseOperationsMixin
from .backends.mysql import \
    ExtendedDatabaseWrapper as MySQLExtendedDatabaseWrapper
from .backends.postgresql import \
    ExtendedDatabaseWrapper as PostgreSQLExtendedDatabaseWrapper
from .backends.sqlite3 import \
    ExtendedDatabaseWrapper as SQLite3ExtendedDatabaseWrapper


def patch_operations_class(original_ops_class):
    if issubclass(original_ops_class, ExtendedDatabaseOperationsMixin):
        return original_ops_class

    class DatabaseOperations(ExtendedDatabaseOperationsMixin, original_ops_class):
        pass
    return DatabaseOperations


def patch_wrapper_class(original_database_wrapper_class):
    def _extend_wrapper(extended_wrapper_class):
        original_database_wrapper_class.operators.update(extended_wrapper_class.operators)
        original_database_wrapper_class.pattern_ops.update(extended_wrapper_class.pattern_ops)
        original_database_wrapper_class.regex_synonyms = extended_wrapper_class.regex_synonyms

    if original_database_wrapper_class.vendor == 'mysql':
        _extend_wrapper(MySQLExtendedDatabaseWrapper)
    elif original_database_wrapper_class.vendor in ['postgresql', 'redshift']:
        _extend_wrapper(PostgreSQLExtendedDatabaseWrapper)
    elif original_database_wrapper_class.vendor == 'sqlite':
        _extend_wrapper(SQLite3ExtendedDatabaseWrapper)
    return original_database_wrapper_class


def patch_load_backend():
    def load_backend(backend_name):
        backend = original_load_backend(backend_name)
        backend.DatabaseWrapper.ops_class = patch_operations_class(backend.DatabaseWrapper.ops_class)
        backend.DatabaseWrapper = patch_wrapper_class(backend.DatabaseWrapper)
        return backend

    original_load_backend = utils.load_backend
    utils.load_backend = load_backend



class ExtendedDatabaseWrapper(object):
    operators = {
        # For negates
        'neexact': '<> %s',
        'neiexact': 'NOT LIKE %s',
        'necontains': 'NOT LIKE BINARY %s',
        'neicontains': 'NOT LIKE %s',
        'neregex': 'NOT REGEXP BINARY %s',
        'neiregex': 'NOT REGEXP %s',
        'nestartswith': 'NOT LIKE BINARY %s',
        'neendswith': 'NOT LIKE BINARY %s',
        'neistartswith': 'NOT LIKE %s',
        'neiendswith': 'NOT LIKE %s',
        # For exregex
        'exregex': 'REGEXP BINARY %s',
        'exiregex': 'REGEXP %s',
        'neexregex': 'NOT REGEXP BINARY %s',
        'neexiregex': 'NOT REGEXP %s',
    }

    pattern_ops = {
        'necontains': "NOT LIKE BINARY CONCAT('%%', {}, '%%')",
        'neicontains': "NOT LIKE CONCAT('%%', {}, '%%')",
        'nestartswith': "NOT LIKE BINARY CONCAT({}, '%%')",
        'neistartswith': "NOT LIKE CONCAT({}, '%%')",
        'neendswith': "NOT LIKE BINARY CONCAT('%%', {})",
        'neiendswith': "NOT LIKE CONCAT('%%', {})",
    }

    regex_synonyms = {
        '\\<': '[[:<:]]',
        '\\>': '[[:>:]]',
        '\\d': '[[:digit:]]',
        '\\D': '[^[:digit:]]',
        '\\w': '[[:alnum:]_]',
        '\\W': '[^[:alnum:]_]',
        '\\s': '[[:space:]]',
        '\\S': '[^[:space:]]',
    }



#!/usr/bin/env python

"""The setup script."""

from setuptools import (
    find_packages,
    setup,
)

import lookup_extensions

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Django>=1.11',
]

test_requirements = [
    'django-environ',
    # TODO: Put package test requirements here
]

setup(
    name='django-lookup-extensions',
    version=lookup_extensions.__version__,
    description="Django lookup extensions use NOT query.",
    long_description=readme + '\n\n' + history,
    author=lookup_extensions.__author__,
    author_email=lookup_extensions.__email__,
    url='https://github.com/uncovertruth/django-lookup-extensions',
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='django, lookup extensions, filter',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests.runtests.runtests',
    tests_require=test_requirements,
)



"""Top-level package for Django lookup extensions."""

__author__ = """UNCOVER TRUTH Inc."""
__email__ = 'develop@uncovertruth.co.jp'
__version__ = '0.0.2'

default_app_config = 'lookup_extensions.apps.LookupExtensionsConfig'
